% camp.m

% Campbell's JME 1994 RBC model with variable labor supply and
% separable utility.

addpath /mq/home/m1hxl00/matlab/redssolds/tools;
% ****************************************************************
% [1] Structural parameters
% ****************************************************************

a = 0.667;
r = 0.015;
g = 0.005;
d = 0.025;
s = 1;
sn = 1;
nbar = 0.33;

% Composite parameters
lam1 = (1+r)/(1+g);
lam2 = a*(r+d)/((1-a)*(1+g));
lam3 = a*(r+d)/(1+r);
nu = ((1-nbar)*sn)/(nbar+(1-a)*(1-nbar)*sn);

% AR coefficient and standard deviation for technology shock
phi = 0.979;
sd = 0.007; % Cooley & Prescott p. 22

% ****************************************************************
% [2] Specify system A*Ey(t+1|t) = B*y(t) + C*x(t) in the
% notation of REDS-SOLDS.
% ****************************************************************

% index for elements of y(t):

iY = 1;	    % y(t)
iC = 2;	    % c(t)
iN = 3;     % n(t)
iR = 4;	    % r(t)
iK = 5;	    % k(t)
iZ = 6;	    % z(t-1)

% index for elements of x(t):

iEPS = 1;

% number of endogenous variables, predetermined variables
% and shocks:

NY = 6;
NX = 1;
NK = 2;

% initializing coefficient matrices

Ae = zeros(NY);
Be = zeros(NY);
Ce = zeros(NY, NX);

% *******************************************************
% [2] Setting up structural equations:
% *******************************************************

% Definition of output

Ae(1, iZ) = a;
Be(1, iY) = 1;
Be(1, iN) = -a;
Be(1, iK) = a-1;

% IS equation

Ae(2, iC) = 1;
Ae(2, iN) = -s*lam3;
Ae(2, iK) = s*lam3;
Ae(2, iZ) = -phi*s*lam3;
Be(2, iC) = 1;

% Intratemporal FOC

Ae(3, iZ) = nu*a;
Be(3, iK) = -nu*(1-a);
Be(3, iN) = 1;
Be(3, iC) = nu;

% Ex post real rate

Ae(4, iZ) = lam3;
Be(4, iR) = 1;
Be(4, iN) = -lam3;
Be(4, iK) = lam3;

% Capital

Ae(5, iK) = 1;
Ae(5, iZ) = -lam2;
Be(5, iC) = 1-lam1-lam2;
Be(5, iK) = lam1;
Be(5, iN) = lam2;

% Technology shock

Ae(6, iZ) = 1;
Be(6, iZ) = phi;
Ce(6, iEPS) = 1;

% ****************************************************************
% [3] Call to REDS-SOLDS for the system specified above
% ****************************************************************

A = Ae;
B = Be;
C = Ce;

reds

solds

% ****************************************************************
% [4] Impulse responses to technology shock
% ****************************************************************

nir = 21;
lead = 0:(nir-1);

Y1e = irf(iEPS, nir, D, F, G, H);
% Use this line to reproduce circled lines in King & Rebelo, Fig 11
% Y1e = Y1e*1/a; 

figure('Name', 'Responses to technology shock')

subplot(311), plot(lead, Y1e(iY, :)), title('Output')
subplot(312), plot(lead, Y1e(iN, :)), title('Labor')
subplot(313), plot(lead, 400*Y1e(iR, :)), title('Real Rate (BPs)')

% ****************************************************************
% [5] Variances of endogenous variables
% ****************************************************************

VCVk = vcv(sd^2, G, H);
VCVy = D*VCVk*D' + sd^2*F*F';
disp(' ')
disp('Variances of y, c, n, r, k, z ')
disp(' ')
disp( VCVy );
